# sui-test-contract

> Useful tooling for defining contract tests (based on [Pact](https://docs.pact.io/)) that will generate contract documents. It will also allow us to publish such documents into a defined [Pact Broker](https://docs.pact.io/pact_broker).

## Dependencies

This package depends on the API mocking tool [MSW](https://mswjs.io/). That means all the mocking requests should be handled by such tool, and the recommendation is to share the mocks along your whole app, so they should be placed in a unique folder named `mocks`, for example, in the root of your project.

## Environment

These kind of contract test are intended to be executed in a server side environment with [Mocha](https://mochajs.org/) and [Chai](https://www.chaijs.com/). It's recommended to use the package [@s-ui/test](https://github.com/SUI-Components/sui/tree/master/packages/sui-test) for executing them.

## Using the setup function

In order to start using the contract tests in your app, you'll need to execute the `setupContractTests` function passing the needed parameters. It will allow you to create the interactions, test them and finally it will generate the contract documents (in the path: `contract/documents` by default).

It's important to know that **YOU'LL NEED TO EXECUTE THIS FUNCTION IN A TEST FILE** (e.g.: `consumerSpec.js`)

Here you have a detailed example:

```js
// This file should be placed in a path like:
// `contract/test/foo/consumerSpec.js`
import {expect} from 'chai'
import {rest} from 'msw'
import {FetcherFactory} from '@s-ui/domain'
import {setupContractTests} from '@s-ui/test-contract'

const fetcher = FetcherFactory.httpFetcher({config: {}})
const consumer = 'test-consumer'
const applesResponse = [
  {color: 'red', type: 'Fuji'},
  {color: 'green', type: 'Granny Smith'}
]

setupContractTests({
  apiUrl: 'http://localhost:8181',
  consumer,
  fetcher,
  providers: {
    'test-provider': [
      {
        endpoint: '/apples',
        description: 'A request for getting some apples',
        state: 'I have some apples',
        handler: rest.get('http://localhost:8181/apples', (req, res, ctx) =>
          res(ctx.status(200), ctx.json(applesResponse))
        ),
        response: applesResponse
      }
    ]
  }
})
```

### Setup options

| Parameter | Required | Type     | Default | Description                                                  |
| --------- | -------- | -------- | ------- | ------------------------------------------------------------ |
| server    | false    | `object` |         | server provided by msw - a server or worker must be provided |

### Provider interaction options

        description,
        endpoint,
        query,
        body,
        method = 'get',
        handler,
        options,
        response

## Publishing the contracts

When you have your contract documents generated (e.g.: `contract/documents/test-consumer-test-provider-123456789.json`), you'll need to publish them to the [Pact Broker](https://docs.pact.io/pact_broker), the place where providers (API Backend) will validate their own tests against the contracts.

You just need to run the following command:

```bash
sui-test-contract publish --broker-url "https://my-contract-tests-broker.com"
```
